﻿#if 1
#include <opencv2/imgproc/imgproc.hpp>  // Gaussian Blur
#include <opencv2/highgui/highgui.hpp>  // OpenCV window I/O
#include <stdlib.h>
#include <stdio.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <string.h>
#include <ctype.h>
#include "tdr_ocr.h"
#include "ini/SimpleIni.h"
#include "iostream"

using namespace std;
//using namespace cv;

extern int g_debug_show;
static int errcount=0;
int total_count=0;
int total_ok_count=0;

bool isDebugShow();
void setDebugShow(int debug);
void get_extension(const char *file_name,char *extension);
void process_default();
int  process_onefile(char * fn);
void process_dir(char* dir_fn, int subdir);
void tdr_set_log_dummy(int onoff);
int full_image_scan = 0;
int select_num=0;
int use_cuda=0;

void showOrgImage(cv::Mat &img)
{
   if(isDebugShow()) {
     cv::Mat showmat;
     int resize_x=1024;
     int resize_y=(float)img.rows/(float)img.cols*1024;
     cv::resize(img, showmat,  cv::Size(resize_x, resize_y), 0, 0, cv::INTER_NEAREST);
     cv::imshow("src", showmat);
     cv::waitKey(1);
   }
}

int main_process(char * fn, int isdir)
{
   cv::Mat img, img_clone;
   int ret=-1;

   img = cv::imread(fn,1);
   if(img.empty()){
         printf("%s is empty\n", fn);
         return -1;
   }
   img_clone=img.clone();
   total_count++;

   if (full_image_scan == 1)
   {
             vector<BOX_RESULT> result;
             int count=crnn_ocr(img, 
                 0,//1600, //int maxSideLen, 注: 值为0时，表示不做缩放处理
                 0,0,//22,50, //int minHeight, int maxHeight, box高度限制条件， 注: 值为0时，表示不限制
                 0,0,//160,250,//int minWidth, int maxWidth, box宽度限制条件, 注: 值为0时，表示不限制
                 0.4, //float box_confidence,
                 (char *)"全部", //char * key,
                 (char *)"交通银行|交道银行|交通銀行|上信赢通|MIZUHO|金融@家|建设银行|Tendy|Tendiy|Tenay|中国电子口岸|中国邮政储蓄银行|ICBC|iCBC|浦发银行|浦发村镇银行|昆仑银行|中国农业银行|citi|cifi|cif|cfi|兴业银行|工商银行|招商银行|中国银行|中信银行|贵安公积金", //char * regex_key  可使用正则表达式
                 result
                 );
             for(int i=0; i<result.size(); i++){
                for(int j=0; j<4;j++){
                    printf("%02d: %4d,%4d: ", result[i].image_id, result[i].boxPoint[j].x,result[i].boxPoint[j].y);
                }
                printf("%s ===> %s \n", result[i].result_txt.c_str(), result[i].valid==1?"OK":"NG");
             }
             if(count<10) errcount++;
             if(count>0) total_ok_count += count;
             ret=0;
    }
    else
    {
      //showOrgImage(img);

      CSimpleIniA ini;
      CSimpleIniA::TNamesDepend  a_names;
      ini.Reset();
      SI_Error rc = ini.LoadFile("box.ini");
      if (rc < 0) {
        return -1;
      } 
      ini.GetAllSections(a_names);
      a_names.sort(typename CSimpleIniA::Entry::LoadOrder());
      std::list<CSimpleIniA::Entry>::iterator it;
      int total_num=0;
      for (it = a_names.begin(); it != a_names.end(); ++it) {
          total_num++;
          printf("%02d : %s\n", total_num, (*it).pItem);
      }
      if(select_num<=0){
           printf("请输入KEY模板的序号: ");
           cin >> select_num;
      }
      if(select_num<=0 || select_num>total_num){
           printf("序号不存在\n");
           return 0;
      }
      it=a_names.begin();
      std::advance(it, select_num-1);
      char * select_keyname = (char *)(*it).pItem;
      printf("your select is %d : %s\n\n", select_num, select_keyname);


      ret = ResetKeyParam((char *)"box.ini", select_keyname);
      if (ret == 0) {
        vector<BOX_RESULT> result;
        int cnt = ScanImage(img, result);
        ret = cnt;
        if (cnt < 0) {
          printf("ScanImage ret error = %d.\n", ret);
        } else {
          printf("cnt=%d; result.size()=%lu.\n ", cnt, result.size());

          for (int i = 0; i < result.size(); i++) {
            if (result[i].valid == 1) {
              printf("%02d: ", result[i].image_id);
              for (int j = 0; j < 4; j++) {
                printf("%4d,%4d: ", result[i].boxPoint[j].x, result[i].boxPoint[j].y);
              }
            }
            printf("%s ===> %s \n", result[i].result_txt.c_str(),
                   result[i].valid == 1 ? "OK" : "NG");
          }
        }
      } else
        printf("ResetKeyParam ret error = %d.\n", ret);
    }


   //printf("ocr return = %d \n", ret);
   printf("--------------------\n\n");

   //printf("error count = %d\n", errcount);
#ifdef TDR_TRACE
  if(ret>=0)
  if(isDebugShow()) cv::waitKey(0);
#endif

   return 0;
}


int init()
{
   int ret;
   if(use_cuda==1)
    ret=tdrocr_init("../models", "cuda",full_image_scan);
   else
    ret=tdrocr_init("../models", "v4", full_image_scan);
   return ret;
}

int uninit()
{
   tdrocr_unInit();
   return 0;
}

int main(int argc, char* argv[]) {

    int ret;
    int i;
    std::string fn1;
    int include_subdir=0;
    if(argc<2){
       printf(
"Usages: ./tdrocr -m0/-m1 -r -s0/-s1 [file]/[dir] \n\
-r: include_subdirectory \n\
-s0: no_debug_output, -s1 debug_output \n\
\n");
       return 0;
    }

    setDebugShow(0);
    tdr_set_log_dummy(0);

    for(i=1; i< argc; i++)
    {
        if( strcmp(argv[i],"-r")==0 )
        {
            include_subdir=1;
        }
        else if( strcmp(argv[i],"-s0" )==0 )
        {
            setDebugShow(0);
        }
        else if( strcmp(argv[i],"-s1" )==0 )
        {
            setDebugShow(1);
        }
        else if( strcmp(argv[i],"-cuda" )==0 )
        {
            use_cuda=1;
        }
        else if (strcmp(argv[i], "-a") == 0)
        {
            //自动进行全图识别，不依赖ini文件中定义的文字框
            full_image_scan = 1;
        }
        else if (strncmp(argv[i], "-k", 2) == 0)
        {
            char select_num_s[5];
            char * p = argv[i] + 2;
            memset(select_num_s,0x00, 5);
            for(int j=0; j<2; j++){
               if(*p==0x00) break;
               select_num_s[j]= *p;
               p++;    
            }
            select_num = atoi(select_num_s);
        }
        else if( strcmp(argv[i],"-d0" )==0 )
        {
            tdr_set_log_dummy(1);
        }
        else if( strcmp(argv[i],"-d1" )==0 )
        {
            tdr_set_log_dummy(0);
        }
        else{
            fn1=std::string(argv[i]);
        }
    }

    if(fn1.length()==0) {
         printf("no filename given. \n");
         return 0;
    }

    ret = init();
//sleep(1000000000);
    if(ret!=0) {
        printf("init fail %d, exit!\n", ret);
        return ret;
    }

 

    struct stat st;
    if((fn1.length()>0) && stat(fn1.c_str(),&st) == 0)
    {
        if(S_ISREG(st.st_mode))
        {
            process_onefile((char *)fn1.c_str());
        }
        else {
            process_dir((char *)fn1.c_str(), include_subdir);
        }
    }

    printf(">>>> error/total = %d/%d , total_ok = %d <<<<\n", errcount, total_count, total_ok_count);
    uninit();
    printf("end\n");
    return 0;
}

void get_extension(const char *file_name,char *extension)
{
    int found=-1;
    int i;
    for(i=strlen(file_name)-1; i>=0; i--)
    {
        if(file_name[i]=='.'){
            found=i;
            break;
        }
        else if(file_name[i]=='/')
        {
           break;
        }
    }

    if(found>=0)
    strcpy(extension,file_name+found+1);
    else
    strcpy(extension,"\0");
    for(i=0; i<strlen(extension); i++)
    {
       extension[i]=tolower(extension[i]);
    }
}

int  process_onefile(char * fn)
{
    char ext_name[128];
    get_extension(fn, ext_name);
    if(strlen(ext_name)==0) return -1;
    if( strcmp(ext_name,"jpg")==0 || strcmp(ext_name,"bmp")==0 || strcmp(ext_name,"png")==0 || strcmp(ext_name,"jpeg")==0 || strcmp(ext_name,"tiff")==0 || strcmp(ext_name,"webp")==0)
    {
printf("\n-----------------------------------------\n");
          printf("%s\n", fn);
printf("-----------------------------------------\n");
          main_process(fn,0);
          return 0;
    }
    return -1;
}

void process_dir(char* dir_fn, int subdir)
{
   DIR* dir = opendir(dir_fn);
   struct dirent* dirp;
   char filename1[512];
   if(dir == NULL)
   {
      return;
   }

   DIR * mydir;
   if((mydir= opendir("output"))==NULL)
       mkdir("output",S_IRWXU|S_IRWXG|S_IRWXO);
   else closedir(mydir);

   while((dirp = readdir(dir)) != NULL)
   {
       if(strcmp(dirp->d_name, ".")==0||strcmp(dirp->d_name, "..")==0||strcmp(dirp->d_name, ".DS_Store")==0)
           continue;
       if(dirp->d_type == DT_REG || dirp->d_type == DT_DIR){
           sprintf(filename1, "%s/%s", dir_fn, dirp->d_name);
           char ext_name[128];
           get_extension(filename1, ext_name);
           if( strcmp(ext_name,"jpg")==0 || strcmp(ext_name,"bmp")==0 || strcmp(ext_name,"png")==0 || strcmp(ext_name,"jpeg")==0 || strcmp(ext_name,"tiff")==0 || strcmp(ext_name,"webp")==0 )
           {
printf("\n-----------------------------------------\n");
              printf("%s\n", filename1);
printf("-----------------------------------------\n");
               main_process(filename1,1);

           }
           else if(subdir==1){
               process_dir(filename1, subdir);
           }
       }
   }

   closedir(dir);
}


#endif
